﻿using EsHelperCore.Dtos;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.ObjectPool;
using Nest;
using System.Buffers;
using System.Net;
using System.Text;

namespace EsHelperCore.Controllers
{
    [ApiController]
    [Route("api")]
    public class EsHelperController : ControllerBase
    {
        private readonly IElasticClient _elasticClient;

        private readonly ILogger _logger;
        public EsHelperController(ILogger<EsHelperController> logger, IElasticClient elasticClient)
        {
            _elasticClient = elasticClient;
            _logger = logger;
        }

        /// <summary>
        /// 添加文件到ES中
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        [HttpPost]
        [Route("add")]
        public async Task<String> AddFileToEs([FromForm] AddFileToEsInput input)
        {
            var bytes1 = System.IO.File.ReadAllBytes(input.Path);

            byte[] bytes = null;
            try
            {
                using FileStream stream = new FileStream(input.Path, FileMode.Open, FileAccess.Read);

                var byteLength = (int)stream.Length;
                bytes = ArrayPool<byte>.Shared.Rent(byteLength);
                stream.Read(bytes, 0, byteLength);

                string str = Convert.ToBase64String(bytes,0, byteLength);

                IndexDto dto = new IndexDto()
                {
                    addtime = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss"),
                    adduser = "admin",
                    data = str,
                    filename = input.FileName,
                    id = input.Id,
                    path = input.Path,
                    title = input.Title,
                };
                 
                var response = await _elasticClient.IndexAsync(dto, x => x.Pipeline("file_attachment"));

                if (response.ServerError == null)
                {
                    _logger.LogInformation($"添加文件成功 {input.Path} {response.Result.ToString()}");
                }
                else
                {
                    _logger.LogError($"添加文件失败 {input.Path} {response.ToString()}");
                }

                return response.Result.ToString();
            }
            finally
            {
                if (bytes != null)
                    ArrayPool<byte>.Shared.Return(bytes);
            }
        }
    }
}
